@@ -43,7 +43,7 @@ static int lnvm_open(void)
 	if (fd < 0) {
 		printf("Failed to open LightNVM mgmt interface\n");
 		perror(dev);
-		exit(errno);
+		return fd;
 	}
 
 	return fd;
@@ -60,6 +60,8 @@ int lnvm_do_init(char *dev, char *mmtype)
 	int fd, ret;
 
 	fd = lnvm_open();
+	if (fd < 0)
+		return fd;
 
 	memset(&init, 0, sizeof(struct nvm_ioctl_dev_init));
 	strncpy(init.dev, dev, DISK_NAME_LEN);
@@ -91,6 +93,8 @@ int lnvm_do_list_devices(void)
 	int fd, ret, i;
 
 	fd = lnvm_open();
+	if (fd < 0)
+		return fd;
 
 	ret = ioctl(fd, NVM_GET_DEVICES, &devs);
 	if (ret)
@@ -118,6 +122,8 @@ int lnvm_do_info(void)
 	int fd, ret, i;
 
 	fd = lnvm_open();
+	if (fd < 0)
+		return fd;
 
 	memset(&c, 0, sizeof(struct nvm_ioctl_info));
 	ret = ioctl(fd, NVM_INFO, &c);
@@ -141,12 +147,14 @@ int lnvm_do_info(void)
 }
 
 int lnvm_do_create_tgt(char *devname, char *tgtname, char *tgttype,
-						int lun_begin, int lun_end)
+					int lun_begin, int lun_end, int flags)
 {
 	struct nvm_ioctl_create c;
 	int fd, ret;
 
 	fd = lnvm_open();
+	if (fd < 0)
+		return fd;
 
 	strncpy(c.dev, devname, DISK_NAME_LEN);
 	strncpy(c.tgtname, tgtname, DISK_NAME_LEN);
@@ -155,6 +163,7 @@ int lnvm_do_create_tgt(char *devname, char *tgtname, char *tgttype,
 	c.conf.type = 0;
 	c.conf.s.lun_begin = lun_begin;
 	c.conf.s.lun_end = lun_end;
+	c.flags = flags;
 
 	ret = ioctl(fd, NVM_DEV_CREATE, &c);
 	if (ret)
@@ -170,6 +179,8 @@ int lnvm_do_remove_tgt(char *tgtname)
 	int fd, ret;
 
 	fd = lnvm_open();
+	if (fd < 0)
+		return fd;
 
 	strncpy(c.tgtname, tgtname, DISK_NAME_LEN);
 	c.flags = 0;
@@ -190,6 +201,8 @@ int lnvm_do_factory_init(char *devname, int erase_only_marked,
 	int fd, ret;
 
 	fd = lnvm_open();
+	if (fd < 0)
+		return fd;
 
 	memset(&fact, 0, sizeof(struct nvm_ioctl_dev_factory));
 
